import osirisl1services.readlevel1 as readlevel1
from skretrieval.core import OpticalGeometry
from skretrieval.core.radianceformat import RadianceGridded
import osirisl1services.services as services
import osirisl1services.spatialrotation as sr
import sasktran as sk
from copy import copy
import numpy as np


class Osiris(object):
    def __init__(self, scan_no, load_psf=False, apply_rsas=True, load_invalid=False, max_tangent_altitude=100000):
        valid = not load_invalid
        self.spectrograph_data = readlevel1.open_level1_spectrograph(scan_no, load_psf=load_psf, apply_rsas=apply_rsas,
                                                                     valid=valid)

        self.spectrograph_data['osiris_look_ecef'] = self.spectrograph_data.l1.look_ecef
        self.spectrograph_data['osiris_position_ecef'] = self.spectrograph_data.l1.position_ecef

        self.scan_direction = self.spectrograph_data.l1.altitude.values[0] < self.spectrograph_data.l1.altitude.values[-1]
        if self.scan_direction:
            self.scan_direction = 1
        else:
            self.scan_direction = -1

        if self.scan_direction == -1:
            self.spectrograph_data = self.spectrograph_data.sel(mjd=self.spectrograph_data.mjd.values[::-1])

        good_alts = self.spectrograph_data.l1.altitude.values < max_tangent_altitude

        self.spectrograph_data = self.spectrograph_data.isel(mjd=good_alts)

    def sasktran_geometry(self):
        geometry = sk.Geometry()

        for mjd, sliced in self.spectrograph_data.groupby('mjd'):
            pos = sliced.osiris_position_ecef.values.flatten()
            look = sliced.osiris_look_ecef.values.flatten()

            geometry.lines_of_sight.append(sk.LineOfSight(observer=pos, look_vector=look, mjd=mjd))

        geometry.lines_of_sight = geometry.lines_of_sight[::self.scan_direction]

        return geometry

    def optical_axis(self):
        axis = []

        for mjd, sliced in self.spectrograph_data.groupby('mjd'):
            pos = sliced.osiris_position_ecef.values.flatten()
            look = sliced.osiris_look_ecef.values.flatten()

            local_up = pos / np.linalg.norm(pos)

            axis.append(OpticalGeometry(observer=np.array(pos), look_vector=np.array(look), local_up=local_up, mjd=mjd))

        axis = axis[::self.scan_direction]

        return axis

    def skretrieval_l1(self, remove_sun=False):
        new_data = self.spectrograph_data.copy(deep=True)

        mjd = new_data.mjd.values

        new_data.mjd.values = np.arange(0, len(mjd))

        new_data = new_data.rename({'osiris_look_ecef': 'los_vectors', 'osiris_position_ecef': 'observer_position',
                                    'data': 'radiance', 'mjd': 'los',
                                    'error': 'radiance_noise'})

        new_data['mjd'] = (['los'], mjd)

        if remove_sun:
            solar_spectrum = sk.SolarSpectrum()

            irradiance = solar_spectrum.irradiance(new_data.wavelength.values, mjd=np.nanmean(mjd), fwhm=new_data.fwhm.values)

            new_data['radiance'] /= irradiance
            new_data['radiance_noise'] /= irradiance

        new_data = new_data.swap_dims({'pixel': 'wavelength'})
        return RadianceGridded(new_data)

if __name__ == "__main__":
    os = Osiris(6432012)

    geo = os.sasktran_geometry()

    l1_data = os.skretrieval_l1(remove_sun=True)

    axis = os.optical_axis()

    print(axis)