from skretrieval.retrieval import RetrievalTarget
import numpy as np
import sasktran as sk
from skretrieval.core.radianceformat import RadianceBase, RadianceGridded
from copy import copy
import logging


class OSIRISAlbedoRetrieval(RetrievalTarget):
    def __init__(self, atmosphere: sk.Atmosphere, high_alt: float,  ret_wavel):
        self._atmosphere = atmosphere
        self._ret_wavel = ret_wavel
        self._high_alt = high_alt
        self._ref_range = 5000
        self._albedo = copy(atmosphere.brdf.albedo)
        self._smallest_albedo = 1e-3
        self._largest_albedo = 1
        self._zero_meas_vector = 0

    def configure_from_model(self, forward_model):
        old_brdf = copy(self._atmosphere.brdf)
        # Set the albedo to 0 to get the meas vector with albedo 0
        self._atmosphere.brdf = 0.0
        rad = forward_model.calculate_radiance()
        self._zero_meas_vector = None
        self._zero_meas_vector = self.measurement_vector(rad)
        self._atmosphere.brdf = old_brdf

    def measurement_vector(self, l1_data: RadianceBase):
        if not isinstance(l1_data, RadianceGridded):
            raise ValueError('Class Albedo Retrieval only supports data in the form RadianceGridded')

        result = dict()

        tangent_locations = l1_data.tangent_locations()
        l1_data.data['tangent_altitude'] = tangent_locations.altitude

        triplet_values = l1_data.data['radiance'].where((l1_data.data['tangent_altitude'] > self._high_alt - self._ref_range) & (l1_data.data['tangent_altitude'] < self._high_alt), drop=False).sel(wavelength=self._ret_wavel, method='nearest')

        result['y'] = float(np.log(triplet_values).mean(dim='los').values)

        if self._zero_meas_vector:
            result['jacobian'] = (result['y'] - self._zero_meas_vector['y']) / float(self.state_vector())
            result['jacobian'] = result['jacobian'].reshape(1, 1)

        result['y'] = np.array([result['y']])

        return result

    def state_vector(self):
        return np.array([self._albedo])

    def update_state(self, x: np.ndarray):
        self._albedo = np.nanmax([self._smallest_albedo, copy(float(x))])
        self._albedo = np.nanmin([self._largest_albedo, self._albedo])
        self._atmosphere.brdf = copy(self._albedo)
        logging.info("Setting albedo to: {}".format(self._albedo))

    def apriori_state(self):
        return None

    def inverse_apriori_covariance(self):
        return None

    def measurement_wavelengths(self, measurement_l1):
        wavel = measurement_l1.data.sel(wavelength=self._ret_wavel, method='nearest')['wavelength'].values

        return np.array(wavel)

    def rtm_options(self):
        options = dict()
        options['numordersofscatter'] = 50

        return options